#include "at32f403a_407.h"
#include "shift_reg.h"
#include "FreeRTOS.h"
#include "task.h"
#include <stdio.h>
#include <stdbool.h>


#define SH_SPI  SPI3


// PD1 -  ART_nCS_LS_CH16/DAC2
// PC10 - ART_CLK_LS (SPI3_CK remap)
// PC11 - ART_MISO_CH16/CS_LS (SPI3_MISO remap)
// PC12 - ART_MOSI_LS (SPI3_MOSI remap)


// PD0 - ART_nCS_LS_CH712/DAC3
// PD2 - ART_MISO_CH712

void sh_init(void)
{
    spi_init_type spi_init_struct;
    gpio_init_type gpio_initstructure;
    
  
    crm_periph_clock_enable(CRM_IOMUX_PERIPH_CLOCK, TRUE);
    crm_periph_clock_enable(CRM_GPIOC_PERIPH_CLOCK, TRUE);
    crm_periph_clock_enable(CRM_GPIOD_PERIPH_CLOCK, TRUE);
    
    gpio_pin_remap_config(SPI3_GMUX_0011, TRUE);    // Переключить PC10, PC11, PC12
    
    // SCK
    gpio_initstructure.gpio_out_type       = GPIO_OUTPUT_PUSH_PULL;  
    gpio_initstructure.gpio_pull           = GPIO_PULL_DOWN;  
    gpio_initstructure.gpio_mode           = GPIO_MODE_MUX;  
    gpio_initstructure.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
    gpio_initstructure.gpio_pins           = GPIO_PINS_10;
    gpio_init(GPIOC, &gpio_initstructure);
  
    // MISO
    gpio_initstructure.gpio_pull           = GPIO_PULL_UP;  
    gpio_initstructure.gpio_mode           = GPIO_MODE_INPUT;  
    gpio_initstructure.gpio_pins           = GPIO_PINS_11;
    gpio_init(GPIOC, &gpio_initstructure);
    
    // MOSI
    gpio_initstructure.gpio_pull           = GPIO_PULL_UP;  
    gpio_initstructure.gpio_mode           = GPIO_MODE_MUX; 
    gpio_initstructure.gpio_pins           = GPIO_PINS_12;
    gpio_init(GPIOC, &gpio_initstructure);
       
    // CS
    gpio_initstructure.gpio_out_type       = GPIO_OUTPUT_PUSH_PULL;  
    gpio_initstructure.gpio_pull           = GPIO_PULL_NONE;  
    gpio_initstructure.gpio_mode           = GPIO_MODE_OUTPUT;  
    gpio_initstructure.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
    gpio_initstructure.gpio_pins           = GPIO_PINS_1;
    gpio_init(GPIOD, &gpio_initstructure); 
    
    
    crm_periph_clock_enable(CRM_SPI3_PERIPH_CLOCK, TRUE);
    
    spi_default_para_init(&spi_init_struct);
    spi_init_struct.transmission_mode = SPI_TRANSMIT_FULL_DUPLEX;
    spi_init_struct.master_slave_mode = SPI_MODE_MASTER;
    spi_init_struct.mclk_freq_division = SPI_MCLK_DIV_32; //SPI_MCLK_DIV_2;
    spi_init_struct.first_bit_transmission = SPI_FIRST_BIT_MSB;
    spi_init_struct.frame_bit_num = SPI_FRAME_8BIT;
    spi_init_struct.clock_polarity = SPI_CLOCK_POLARITY_HIGH;//SPI_CLOCK_POLARITY_LOW;
    spi_init_struct.clock_phase = SPI_CLOCK_PHASE_2EDGE;
    spi_init_struct.cs_mode_selection = SPI_CS_SOFTWARE_MODE;
    
    spi_init(SPI3, &spi_init_struct);
 
    spi_hardware_cs_output_enable(SPI3, FALSE);
    
    spi_enable(SPI3, TRUE);
}


//
void sh_test(uint8_t val)
{
    uint8_t ret;
    
#if 0    
    gpio_bits_set(GPIOD, GPIO_PINS_1);
    gpio_bits_reset(GPIOD, GPIO_PINS_1);
    
    while (spi_i2s_flag_get(SH_SPI, SPI_I2S_TDBE_FLAG) == RESET);
    SH_SPI->dt = val;
  
    while (spi_i2s_flag_get(SH_SPI, SPI_I2S_RDBF_FLAG) == RESET);
    ret = SH_SPI->dt;

/*    
    gpio_bits_set(GPIOD, GPIO_PINS_1);

    vTaskDelay(1);
    
    gpio_bits_reset(GPIOD, GPIO_PINS_1);
*/    
    printf("SH return: %X\r\n", ret);
#endif    
    
    while (spi_i2s_flag_get(SH_SPI, SPI_I2S_TDBE_FLAG) == RESET);
    SH_SPI->dt = val;
  
    while (spi_i2s_flag_get(SH_SPI, SPI_I2S_RDBF_FLAG) == RESET);
    ret = SH_SPI->dt;

    gpio_bits_set(GPIOD, GPIO_PINS_1);
    gpio_bits_reset(GPIOD, GPIO_PINS_1);

    printf("SH return: %X\r\n", ret & 0x3F);
}

//
void sh_delay(void)
{
    for (unsigned int i = 0; i < 1; i++) {}
}